"""
COnfig File MANipulation library.

The :py:class:`File` class provides a dictionary like interface to simple
configuration files.
"""
import collections

from . import mechanics

class File(object):
	"""
	Mapping interface to a configuration file existing on the file system.

	Every action will cause the file to be wholly read, so using `update` to make
	multiple changes is desirable.
	"""
	def __init__(self, path, open = open, config = mechanics.kinds['single_quote']):
		self.config = config
		self.path = path
		self.Open = open

	def __repr__(self):
		return "%s.%s(%r)" %(
			type(self).__module__,
			type(self).__name__,
			self.path
		)

	def _save(self, lines : [str]):
		with self.Open(self.path, 'w') as cf:
			for l in lines:
				cf.write(l)

	def __delitem__(self, k):
		with self.Open(self.path) as cf:
			lines = mechanics.manipulate({k : None}, cf, **config)
		self._save()

	def __getitem__(self, k):
		with self.Open(self.path) as cfo:
			return mechanics.read_config(
				cfo,
				selector = k.__eq__, # filter everything but the key
				**self.config
			)[k]

	def __setitem__(self, k, v):
		self.update({k : v})

	def __iter__(self):
		return self.keys()

	def __len__(self):
		return len(list(self.keys()))

	def get(self, k, alt = None):
		with self.Open(self.path) as cf:
			return mechanics.read_config(cf, selector = k.__eq__, **config).get(k, alt)

	def keys(self):
		return mechanics.read_config(self.Open(self.path), **self.config).keys()

	def values(self):
		return mechanics.read_config(self.Open(self.path), **self.config).values()

	def items(self):
		return mechanics.read_config(self.Open(self.path), **self.config).items()

	def update(self, keyvals):
		"""
		Given a dictionary of settings, apply them to the file.
		"""
		with self.Open(self.path) as cf:
			lines = mechanics.manipulate(keyvals, cf, **self.config)
		self._save(lines)

	def getset(self, keys):
		"""
		Get all the settings in the list of keys.
		Returns a *dictionary* of those keys.

		This does not return commented entries.
		"""
		keys = set(keys)
		with self.Open(self.path) as cfo:
			cfg = mechanics.read_config(
				cfo,
				selector = keys.__contains__,
				**self.config
			)
			for x in (keys - set(cfg.keys())):
				# Always deals with text, so use None to designate absence.
				cfg[x] = None
			return cfg
collections.Mapping.register(File)
##
# vim: ts=3:sw=3:noet:
