from os import linesep
from io import StringIO
from .. import mechanics

pconfig = mechanics.kinds['single_quote']

sample_config_Aroma = \
"""
##
# A sample config file.
##
# This provides a good = test for manipulate.

#shared_buffers = 4500
search_path = window,$user,public
shared_buffers = 2500

port = 5234
listen_addresses = 'localhost'
listen_addresses = '*'
"""

##
# Wining cases are alteration cases that provide
# source and expectations from an alteration.
#
# The first string is the source, the second the
# alterations to make, the and the third, the expectation.
##
winning_cases = [
	(
		# Two top contenders; the first should be altered, second commented.
		"foo = bar"+linesep+"foo = bar",
		{'foo' : 'newbar'},
		"foo = 'newbar'"+linesep+"#foo = bar"
	),
	(
		# Two top contenders, first one stays commented
		"#foo = bar"+linesep+"foo = bar",
		{'foo' : 'newbar'},
		"#foo = bar"+linesep+"foo = 'newbar'"
	),
	(
		# Two top contenders, second one stays commented
		"foo = bar"+linesep+"#foo = bar",
		{'foo' : 'newbar'},
		"foo = 'newbar'"+linesep+"#foo = bar"
	),
	(
		# Two candidates
		"foo = bar"+linesep+"foo = none",
		{'foo' : 'bar'},
		"foo = 'bar'"+linesep+"#foo = none"
	),
	(
		# Two candidates, winner should be the first, second gets comment
		"#foo = none"+linesep+"foo = bar",
		{'foo' : 'none'},
		"foo = 'none'"+linesep+"#foo = bar"
	),
	(
		# Two commented candidates
		"#foo = none"+linesep+"#foo = some",
		{'foo' : 'bar'},
		"foo = 'bar'"+linesep+"#foo = some"
	),
	(
		# Two commented candidates, the latter a top contender
		"#foo = none"+linesep+"#foo = bar",
		{'foo' : 'bar'},
		"#foo = none"+linesep+"foo = 'bar'"
	),
	(
		# Replace empty value
		"foo = "+linesep,
		{'foo' : 'feh'},
		"foo = 'feh'"
	),
	(
		# Comment value
		"foo = bar",
		{'foo' : None},
		"#foo = bar"
	),
	(
		# Commenting after value
		"foo = val this should be commented",
		{'foo' : 'newval'},
		"foo = 'newval' #this should be commented"
	),
	(
		# Commenting after value
		"#foo = val this should be commented",
		{'foo' : 'newval'},
		"foo = 'newval' #this should be commented"
	),
	(
		# Commenting after quoted value
		"#foo = 'val'foo this should be commented",
		{'foo' : 'newval'},
		"foo = 'newval' #this should be commented"
	),
	(
		# Adjacent post-value comment
		"#foo = 'val'#foo this should be commented",
		{'foo' : 'newval'},
		"foo = 'newval'#foo this should be commented"
	),
	(
		# New setting in empty string
		"",
		{'bar' : 'newvar'},
		"bar = 'newvar'",
	),
	(
		# New setting
		"foo = 'bar'",
		{'bar' : 'newvar'},
		"foo = 'bar'"+linesep+"bar = 'newvar'",
	),
	(
		# New setting with quote escape
		"foo = 'bar'",
		{'bar' : "new'var"},
		"foo = 'bar'"+linesep+"bar = 'new''var'",
	),
]

def parseNone(test, line):
	sl = mechanics.parse_line(line, **pconfig)
	if sl is not None:
		test.fail(
			"With line %r, parsed out to %r, %r, and %r, %r, " \
			"but expected None to be returned by parse function." %(
				line, line[sl[0]], sl[0], line[sl[0]], sl[0]
			)
		)

def parseExpect(test, line, key, val):
	line = line %(key, val)
	sl = mechanics.parse_line(line, **pconfig)
	if sl is None:
		test.fail(
			"expecting %r and %r from line %r, " \
			"but got None(syntax error) instead." %(
				key, val, line
			)
		)
	k, v = sl
	if line[k] != key:
		test.fail(
			"expecting key %r for line %r, " \
			"but got %r from %r instead." %(
				key, line, line[k], k
			)
		)
	if line[v] != val:
		test.fail(
			"expecting value %r for line %r, " \
			"but got %r from %r instead." %(
				val, line, line[v], v
			)
		)

def test_parser(test):
	parseExpect(test, "#%s = %s", 'foo', 'none')
	parseExpect(test, "#%s=%s"+linesep, 'foo', 'bar')
	parseExpect(test, " #%s=%s"+linesep, 'foo', 'bar')
	parseExpect(test, '%s =%s'+linesep, 'foo', 'bar')
	parseExpect(test, ' %s=%s '+linesep, 'foo', 'Bar')
	parseExpect(test, ' %s = %s '+linesep, 'foo', 'Bar')
	parseExpect(test, '# %s = %s '+linesep, 'foo', 'Bar')
	parseExpect(test, '\t # %s = %s '+linesep, 'foo', 'Bar')
	parseExpect(test, '  # %s =   %s '+linesep, 'foo', 'Bar')
	parseExpect(test, "  # %s = %s"+linesep, 'foo', "' Bar '")
	parseExpect(test, "%s = %s# comment"+linesep, 'foo', '')
	parseExpect(test, "  # %s = %s # A # comment"+linesep, 'foo', "' B''a#r '")
	# No equality or equality in complex comment
	parseNone(test, ' #i  # foo =   Bar '+linesep)
	parseNone(test, '#bar')
	parseNone(test, 'bar')

def test_config_read(test):
	sample = "foo = bar"+linesep+"# A comment, yes."+linesep+" bar = foo # yet?"+linesep
	d = mechanics.read_config(sample.split(linesep), **pconfig)
	test.fail_if_not_equal(d['foo'], 'bar')
	test.fail_if_not_equal(d['bar'], 'foo')

def test_config_write_read(test):
	strio = StringIO()
	d = {
		'' : "'foo bar'"
	}
	mechanics.write_config(d, strio.write, **pconfig)
	strio.seek(0)

def test_winning_cases(test):
	i = 0
	for before, alters, after in winning_cases:
		befg = (x + linesep for x in before.split(linesep))
		became = ''.join(mechanics.manipulate(alters, befg, **pconfig))
		test.fail_if_not_equal(became.strip(), after)
		i += 1

def test_simple_config_alter(test):
	# Simple set and uncomment and set test.
	strio = StringIO()
	strio.write("foo = bar"+linesep+" # bleh = unset"+linesep+" # grr = 'oh yeah''s'")
	strio.seek(0)
	lines = mechanics.manipulate({'foo' : 'yes', 'bleh' : 'feh'}, strio, **pconfig)
	d = mechanics.read_config(lines, **pconfig)
	test.fail_if_not_equal(d['foo'], 'yes')
	test.fail_if_not_equal(d['bleh'], 'feh')
	test.fail_if_not_equal(''.join(lines).count('bleh'), 1)

def test_aroma(test):
	lines = mechanics.manipulate({
			'shared_buffers' : '800',
			'port' : None
		}, (x + linesep for x in sample_config_Aroma.split('\n')),
		**pconfig
	)
	d = mechanics.read_config(lines, **pconfig)
	test.fail_if_not_equal(d['shared_buffers'], '800')
	test.fail_if_not_equal(d.get('port'), None)

	nlines = mechanics.manipulate({'port' : '1'}, lines, **pconfig)
	d2 = mechanics.read_config(nlines, **pconfig)
	test.fail_if_not_equal(d2.get('port'), '1')
	test.fail_if_not_equal(nlines[:4], lines[:4])

def test_selection(test):
	# Sanity
	red = mechanics.read_config(['foo = bar'+linesep, 'bar = foo'], **pconfig)
	test.fail_if_not_equal(len(red.keys()), 2)

	# Test a simple selector
	red = mechanics.read_config(['foo = bar'+linesep, 'bar = foo'],
		selector = lambda x: x == 'bar', **pconfig)
	rkeys = list(red.keys())
	test.fail_if_not_equal(len(rkeys), 1)
	test.fail_if_not_equal(rkeys[0], 'bar')
	test.fail_if_not_equal(red['bar'], 'foo')

if __name__ == '__main__':
	from dev import libtest; libtest.execmodule()
